const path = require('path')
const fs = require('fs-extra')
const VendorExtracter = require('./extracter')
const chalk = require('chalk')

module.exports = extract

function extract (opts) {
  const entries = Array.isArray(opts.entries) ? opts.entries : []
  const fileModuleFomatter = opts.fileModuleFomatter || (i => i)
  const appendVersion = opts.appendVersion
  const moduleFilter = opts.moduleFilter || (_ => true)

  const tempDirectory = path.join(process.cwd(), '.vext-tmp')
  try {
    fs.ensureDirSync(tempDirectory)
  }
  catch (e) {
    console.log(`临时目录创建失败... ${e.message}`)
    process.exit(1)
  }

  console.log('vendor 列表提取开始')

  const extracter = new VendorExtracter({
    fileModuleFomatter, moduleFilter
  })
  const vendorList = extracter.parse(entries)
  const vendorEntryContent = extracter.output(vendorList)
  const vendorEntryFile = path.join(tempDirectory, 'vendors.js')
  try {
    fs.writeFileSync(vendorEntryFile, vendorEntryContent)
    console.log('vendor 依赖入口已生成完成');
  }
  catch (e) {
    console.log(`写入临时文件失败... ${e.message}`)
    process.exit(1)
  }

  // 获取 vendor 的版本号
  const vendorMap = vendorList.reduce((ret, vendor) => {
    ret[vendor] = appendVersion ? resolveNpmVersion(vendor) : ''
    return ret
  }, {})

  console.log('=======================')
  console.log('webpack 文件生成开始')

  const webpack = require('webpack')
  const FmdTemplatePlugin = require('../webpack-fmd')
  webpack({
    entry: {
      vendors: vendorEntryFile
    },
    output: {
      path: tempDirectory,
      filename: 'pkg.[name].js',
      publicPath: '/static/'
    },
    plugins: [
      new FmdTemplatePlugin({
        prefix: 'gallery/',
        name: 'vendors.js',
        deps: vendorMap
      })
    ]
  }, (err, stats) => {
    if (err) {
      console.log(`webpack 生成文件失败... ${err.message}`)
    }
    else {
      console.log('任务成功')
    }
  })
}

function resolveNpmVersion (moduleName) {
  try {
    const main = require.resolve(moduleName)
    const pkgFile = resolvePkg(main)
    if (pkgFile) {
      const pkg = require(pkgFile)
    }

    return pkg.version
  }
  catch (e) {
    console.log(chalk.red(`[Extracter] 找不到模块 ${moduleName}`));
  }
}

function resolvePkg (main) {
  if (path.basename(main) == 'node_modules' || main == '/') return

  const pkgFile = path.join(main, '../package.json')
  if (fs.existsSync(pkgFile)) return pkgFile
  else {
    return resolvePkg(path.join(main, '..'))
  }
}
